@@ -1,5 +1,5 @@
 /* Zutils - Utilities dealing with compressed files
-   Copyright (C) 2009-2022 Antonio Diaz Diaz.
+   Copyright (C) 2009-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ bool next_filename( std::list< std::string > & filenames,
              i > 1 && input_filename[i-1] == '/'; --i )
           input_filename.resize( i - 1 );	// remove trailing slashes
         struct stat stdot, *stdotp = 0;
-        if( input_filename[0] != '/' )		// relative path
+        if( input_filename[0] != '/' )		// relative file name
           {
           if( input_filename == "." ) input_filename.clear();
           if( stat( ".", &stdot ) == 0 && S_ISDIR( stdot.st_mode ) )
