@@ -1,5 +1,5 @@
 /* Zdiff - decompress and compare two files line by line
-   Copyright (C) 2010-2022 Antonio Diaz Diaz.
+   Copyright (C) 2010-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ void show_help()
                "  -i, --ignore-case                 ignore case differences in file contents\n"
                "  -M, --format=<list>               process only the formats in <list>\n"
                "  -N, --no-rcfile                   don't read runtime configuration file\n"
-               "  -O, --force-format=[<f1>][,<f2>]  force the formats given (bz2,gz,lz,xz,zst)\n"
+               "  -O, --force-format=[<f1>][,<f2>]  force one or both input formats\n"
                "  -p, --show-c-function             show which C function each change is in\n"
                "  -q, --brief                       output only whether files differ\n"
                "  -s, --report-identical-files      report when two files are identical\n"
@@ -89,13 +89,15 @@ void show_help()
                "  -U, --unified=<n>                 same as -u but use <n> lines of context\n"
                "  -v, --verbose                     verbose mode (for --version)\n"
                "  -w, --ignore-all-space            ignore all white space\n"
-               "  -W, --width=<n>                   output at most <n> print columns\n"
+               "  -W, --width=<n>                   output at most <n> print columns (for -y)\n"
                "  -y, --side-by-side                output in two columns\n"
                "      --bz2=<command>               set compressor and options for bzip2 format\n"
                "      --gz=<command>                set compressor and options for gzip format\n"
                "      --lz=<command>                set compressor and options for lzip format\n"
                "      --xz=<command>                set compressor and options for xz format\n"
-               "      --zst=<command>               set compressor and options for zstd format\n" );
+               "      --zst=<command>               set compressor and options for zstd format\n"
+               "\nValid formats for options '-M' and '-O' are 'bz2', 'gz', 'lz', 'xz', 'zst',\n"
+               "and 'un' for uncompressed.\n" );
   show_help_addr();
   }
 
@@ -128,7 +130,7 @@ bool set_fifonames( const std::string filenames[2] )
 
   if( p ) { fifonames[0] = p; fifonames[0] += '/'; }
   else fifonames[0] = "/tmp/";
-  int n = getpid();
+  unsigned n = getpid();
   do fifonames[0] += codes[n % num_codes]; while( n /= num_codes );
   const unsigned pos = fifonames[0].size();
   fifonames[0] += '_';
@@ -186,12 +188,8 @@ bool set_data_feeder( const std::string & filename,
       {
       const int outfd = open( fifoname.c_str(), O_WRONLY | O_BINARY );
       if( outfd < 0 )
-        {
-        if( verbosity >= 0 )
-          std::fprintf( stderr, "%s: Can't open FIFO '%s' for writing: %s\n",
-                        program_name, fifoname.c_str(), std::strerror( errno ) );
-        _exit( 2 );
-        }
+        { show_file_error( fifoname.c_str(), "Can't open FIFO for writing",
+                           errno ); _exit( 2 ); }
       if( dup2( fda[0], STDIN_FILENO ) >= 0 &&
           dup2( outfd, STDOUT_FILENO ) >= 0 &&
           close( fda[0] ) == 0 && close( fda[1] ) == 0 &&
@@ -225,12 +223,8 @@ bool set_data_feeder( const std::string & filename,
       {
       const int outfd = open( fifoname.c_str(), O_WRONLY | O_BINARY );
       if( outfd < 0 )
-        {
-        if( verbosity >= 0 )
-          std::fprintf( stderr, "%s: Can't open FIFO '%s' for writing: %s\n",
-                        program_name, fifoname.c_str(), std::strerror( errno ) );
-        _exit( 2 );
-        }
+        { show_file_error( fifoname.c_str(), "Can't open FIFO for writing",
+                           errno ); _exit( 2 ); }
       if( !feed_data( filename, infd, outfd, magic_data, magic_size ) )
         _exit( 2 );
       if( close( outfd ) != 0 )
@@ -268,7 +262,7 @@ int main( const int argc, const char * const argv[] )
   {
   enum { bz2_opt = 256, gz_opt, lz_opt, xz_opt, zst_opt };
   std::vector< const char * > diff_args;	// args to diff, maybe empty
-  int format_types[2] = { -1, -1 };
+  int format_types[2] = { -1, -1 };		// < 0 means undefined
   program_name = "zdiff";
   invocation_name = ( argc > 0 ) ? argv[0] : program_name;
 
@@ -317,41 +311,39 @@ int main( const int argc, const char * const argv[] )
     const int code = parser.code( argind );
     if( !code ) break;					// no more options
     const char * const pn = parser.parsed_name( argind ).c_str();
-    const std::string & arg = parser.argument( argind );
+    const std::string & sarg = parser.argument( argind );
+    const char * const arg = sarg.c_str();
     switch( code )
       {
       case 'a': diff_args.push_back( "-a" ); break;
       case 'b': diff_args.push_back( "-b" ); break;
       case 'B': diff_args.push_back( "-B" ); break;
       case 'c': diff_args.push_back( "-c" ); break;
-      case 'C': diff_args.push_back( "-C" );
-                diff_args.push_back( arg.c_str() ); break;
+      case 'C': diff_args.push_back( "-C" ); diff_args.push_back( arg ); break;
       case 'd': diff_args.push_back( "-d" ); break;
       case 'E': diff_args.push_back( "-E" ); break;
       case 'h': show_help(); return 0;
       case 'i': diff_args.push_back( "-i" ); break;
-      case 'M': parse_format_list( arg, pn ); break;
+      case 'M': parse_format_list( sarg, pn ); break;
       case 'N': break;
-      case 'O': parse_format_types2( arg, pn, format_types ); break;
+      case 'O': parse_format_types2( sarg, pn, format_types ); break;
       case 'p': diff_args.push_back( "-p" ); break;
       case 'q': diff_args.push_back( "-q" ); break;
       case 's': diff_args.push_back( "-s" ); break;
       case 't': diff_args.push_back( "-t" ); break;
       case 'T': diff_args.push_back( "-T" ); break;
       case 'u': diff_args.push_back( "-u" ); break;
-      case 'U': diff_args.push_back( "-U" );
-                diff_args.push_back( arg.c_str() ); break;
-      case 'v': verbosity = 1; break;
+      case 'U': diff_args.push_back( "-U" ); diff_args.push_back( arg ); break;
+      case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version( DIFF " --version" ); return 0;
       case 'w': diff_args.push_back( "-w" ); break;
-      case 'W': diff_args.push_back( "-W" );
-                diff_args.push_back( arg.c_str() ); break;
+      case 'W': diff_args.push_back( "-W" ); diff_args.push_back( arg ); break;
       case 'y': diff_args.push_back( "-y" ); break;
-      case bz2_opt: parse_compressor( arg, fmt_bz2 ); break;
-      case gz_opt: parse_compressor( arg, fmt_gz ); break;
-      case lz_opt: parse_compressor( arg, fmt_lz ); break;
-      case xz_opt: parse_compressor( arg, fmt_xz ); break;
-      case zst_opt: parse_compressor( arg, fmt_zst ); break;
+      case bz2_opt: parse_compressor( sarg, pn, fmt_bz2 ); break;
+      case gz_opt: parse_compressor( sarg, pn, fmt_gz ); break;
+      case lz_opt: parse_compressor( sarg, pn, fmt_lz ); break;
+      case xz_opt: parse_compressor( sarg, pn, fmt_xz ); break;
+      case zst_opt: parse_compressor( sarg, pn, fmt_zst ); break;
       default : internal_error( "uncaught option." );
       }
     } // end process options
@@ -361,12 +353,10 @@ int main( const int argc, const char * const argv[] )
   setmode( STDOUT_FILENO, O_BINARY );
 #endif
 
-  if( argind >= parser.arguments() )
-    { show_error( "No files given.", 0, true ); return 2; }
-  if( parser.arguments() - argind > 2 )
-    { show_error( "Too many files.", 0, true ); return 2; }
-
   const int files = parser.arguments() - argind;
+  if( files < 1 ) { show_error( "No files given.", 0, true ); return 2; }
+  if( files > 2 ) { show_error( "Too many files.", 0, true ); return 2; }
+
   std::string filenames[2];		// file names of the two input files
   filenames[0] = parser.argument( argind );
   if( files == 2 ) filenames[1] = parser.argument( argind + 1 );
