@@ -1,5 +1,5 @@
 /* Zutils - Utilities dealing with compressed files
-   Copyright (C) 2009-2022 Antonio Diaz Diaz.
+   Copyright (C) 2009-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -110,7 +110,7 @@ int writeblock( const int fd, const uint8_t * const buf, const int size )
   }
 
 
-// Empty filename means stdin.
+// filename == "-" means stdin.
 //
 bool feed_data( const std::string & filename, const int infd, const int outfd,
                 const uint8_t * magic_data, const int magic_size )
@@ -123,8 +123,8 @@ bool feed_data( const std::string & filename, const int infd, const int outfd,
     {
     const int size = readblock( infd, buffer, buffer_size );
     if( size != buffer_size && errno )
-      { const char * const name = filename.empty() ? "-" : filename.c_str();
-        show_file_error( name, "Read error", errno ); return false; }
+      { show_file_error( name_or_stdin( filename.c_str() ), "Read error",
+                         errno ); return false; }
     if( size > 0 && writeblock( outfd, buffer, size ) != size )
       { show_error( "Write error", errno ); return false; }
     if( size < buffer_size ) break;
@@ -183,8 +183,7 @@ bool set_data_feeder( const std::string & filename, int * const infdp,
           close( fda2[0] ) != 0 || close( fda2[1] ) != 0 ||
           !feed_data( filename, old_infd, fda[1], magic_data, magic_size ) )
         _exit( 2 );
-      if( close( fda[1] ) != 0 )
-        { show_close_error(); _exit( 2 ); }
+      if( close( fda[1] ) != 0 ) { show_close_error(); _exit( 2 ); }
       _exit( 0 );
       }
     if( pid < 0 )			// parent
@@ -232,8 +231,7 @@ bool set_data_feeder( const std::string & filename, int * const infdp,
       if( close( fda[0] ) != 0 ||
           !feed_data( filename, old_infd, fda[1], magic_data, magic_size ) )
         _exit( 2 );
-      if( close( fda[1] ) != 0 )
-        { show_close_error(); _exit( 2 ); }
+      if( close( fda[1] ) != 0 ) { show_close_error(); _exit( 2 ); }
       _exit( 0 );
       }
     if( pid < 0 )			// parent
@@ -246,7 +244,7 @@ bool set_data_feeder( const std::string & filename, int * const infdp,
   }
 
 
-// Return format index, or -1 if uncompressed.
+// Return format_index, or -1 if uncompressed.
 //
 int test_format( const int infd, uint8_t magic_data[],
                  int * const magic_sizep )
@@ -255,11 +253,14 @@ int test_format( const int infd, uint8_t magic_data[],
           gzip_magic_size = 2,
           lzip_magic_size = 5,
             xz_magic_size = 5,
-          zstd_magic_size = 4 };
+          zstd_magic_size = 4,
+      compress_magic_size = 2 };
   const uint8_t bzip2_magic[bzip2_magic_size] =
     { 0x42, 0x5A, 0x68 };				// "BZh"
   const uint8_t gzip_magic[gzip_magic_size] =
     { 0x1F, 0x8B };
+  const uint8_t compress_magic[compress_magic_size] =
+    { 0x1F, 0x9D };
   const uint8_t lzip_magic[lzip_magic_size] =
     { 0x4C, 0x5A, 0x49, 0x50, 0x01 };			// "LZIP\001"
   const uint8_t xz_magic[xz_magic_size] =
@@ -277,7 +278,8 @@ int test_format( const int infd, uint8_t magic_data[],
         magic_data[3] >= '1' && magic_data[3] <= '9' &&
         std::memcmp( magic_data + 4, "1AY&SY", 6 ) == 0 )
       return fmt_bz2;
-    if( std::memcmp( magic_data, gzip_magic, gzip_magic_size ) == 0 )
+    if( std::memcmp( magic_data, gzip_magic, gzip_magic_size ) == 0 ||
+        std::memcmp( magic_data, compress_magic, compress_magic_size ) == 0 )
       return fmt_gz;
     if( std::memcmp( magic_data, zstd_magic, zstd_magic_size ) == 0 )
       return fmt_zst;
