@@ -1,5 +1,5 @@
 /* Zutils - Utilities dealing with compressed files
-   Copyright (C) 2009-2022 Antonio Diaz Diaz.
+   Copyright (C) 2009-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,9 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+inline const char * name_or_stdin( const char * const name )
+  { return ( name[0] == '-' && name[1] == 0 ) ? "(stdin)" : name; }
+
 int readblock( const int fd, uint8_t * const buf, const int size );
 int writeblock( const int fd, const uint8_t * const buf, const int size );
 bool feed_data( const std::string & filename, const int infd, const int outfd,
@@ -31,7 +34,7 @@ bool set_data_feeder( const std::string & filename, int * const infdp,
 
 enum { magic_buf_size = 10 };		// >= longest extended magic (bzip2)
 
-// Return format index, or -1 if uncompressed.
+// Return format_index, or -1 if uncompressed.
 //
 int test_format( const int infd, uint8_t magic_data[],
                  int * const magic_sizep );
